#define BLUE_PIN 3
#define RED_PIN 5
#define GREEN_PIN 6

#define PIR_SENSOR 8
#define PHOTO_SENSOR A0
#define BUTTON_PIN 2

#define FADESPEED 3
#define WAIT_DELAY 1000

volatile int lastButtonState = HIGH;
volatile int buttonState = HIGH;
volatile bool isAutomatic = true;

long long lastTime;
long long sensorLastTime;

int red = 0;
int blue = 255;
int green = 0;

int stage;
bool isOn = false;

void setup()
{
  Serial.begin(9600);

  pinMode(BUTTON_PIN, INPUT_PULLUP);
  pinMode(BLUE_PIN, OUTPUT);
  pinMode(RED_PIN, OUTPUT);
  pinMode(GREEN_PIN, OUTPUT);
  pinMode(PIR_SENSOR, INPUT);

  lastTime = millis();
  sensorLastTime = millis();

  attachInterrupt(digitalPinToInterrupt(BUTTON_PIN), button_interrupt, FALLING);

  turnLightsOff();
}

void button_interrupt() {
  isAutomatic = !isAutomatic;
  if (isAutomatic) isOn = false;

  Serial.print( "Changed automatic state: ");
  Serial.println( isAutomatic );
}

void loop()
{
  if ( (millis() - lastTime) > FADESPEED ) {
    // change colors
    if (stage % 3 == 0) {
      red++;
      blue--;

      if (blue <= 0) stage++;
    } else if (stage % 3 == 1) {
      red--;
      green++;

      if (red <= 0) stage++;
    } else {
      green--;
      blue++;

      if (green <= 0) stage++;
    }

    delay(3);

    lastTime = millis();
  }

  if (isAutomatic && ( !isOn || isOn && (millis() - sensorLastTime < WAIT_DELAY) )) {
    int sensorState = digitalRead(PIR_SENSOR);
    int photoValue = analogRead(PHOTO_SENSOR);

    if (sensorState == 1 && photoValue <= 300) {
      turnLightsOn();
      isOn = true;
      sensorLastTime = millis();
    } else {
      turnLightsOff();
      isOn = false;
    }
  } else if (!isAutomatic) {
    turnLightsOn();
    return;
  }
}

void turnLightsOn() {
  analogWrite(BLUE_PIN, blue);
  analogWrite(RED_PIN, red);
  analogWrite(GREEN_PIN, green);
}

void turnLightsOff() {
  analogWrite(BLUE_PIN, 255);
  analogWrite(RED_PIN, 255);
  analogWrite(GREEN_PIN, 255);
}
